#Author-Mike Bax
#Description-Creates an internal involute gear

import adsk.core, adsk.fusion, adsk.cam, traceback
import math

# Globals
_app = adsk.core.Application.cast(None)
_ui = adsk.core.UserInterface.cast(None)
_units = ''

_outsidediameter = adsk.core.ValueCommandInput.cast(None)
_module = adsk.core.ValueCommandInput.cast(None)
_nrteeth = adsk.core.ValueCommandInput.cast(None)
_thickness = adsk.core.ValueCommandInput.cast(None)
_pressureangle = adsk.core.ValueCommandInput.cast(None)


_handlers = []

def run(context):
    try:
        global _app, _ui
        _app = adsk.core.Application.get()
        _ui  = _app.userInterface

        cmdDef = _ui.commandDefinitions.itemById('adskInternalGearPythonScript')
        if not cmdDef:
            # Create a command definition.
           cmdDef = _ui.commandDefinitions.addButtonDefinition('adskInternalGearPythonScript', 'Metric Internal Gear', 'Creates a internal gear component', './resources')
        
        # Connect to the command created event.
        onCommandCreated = GearCommandCreatedHandler()
        cmdDef.commandCreated.add(onCommandCreated)
        _handlers.append(onCommandCreated)
        
        # Execute the command.
        cmdDef.execute()

        # prevent this module from being terminate when the script returns, because we are waiting for event handlers to fire
        adsk.autoTerminate(False)
    except:
        if _ui:
            _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


class GearCommandDestroyHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandEventArgs.cast(args)

            # when the command is done, terminate the script
            # this will release all globals which will remove all event handlers
            adsk.terminate()
        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


# Event handler for the commandCreated event.
class GearCommandCreatedHandler(adsk.core.CommandCreatedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandCreatedEventArgs.cast(args)
            
            # Verify that a Fusion design is active.
            des = adsk.fusion.Design.cast(_app.activeProduct)
            if not des:
                _ui.messageBox('A Fusion design must be active when invoking this command.')
                return()
                

            cmd = eventArgs.command
            cmd.isExecutedWhenPreEmpted = False
            inputs = cmd.commandInputs
            
            
            global _module, _outsidediameter, _imgInputLogo, _nrteeth, _thickness, _pressureangle
            
            _imgInputLogo = inputs.addImageCommandInput('Logo', '', 'resources/BAXEDM_288.png')
            _imgInputLogo.isFullWidth = True
    
            inputs.addTextBoxCommandInput('readonly_textBox', 'Note:', 'This generator requires a metric input. For imperial you need to convert manually to metric first. There is also no input validation (garbage in = garbage out) \n\nVisit https://baxedm.com for info on how to make your own wire-EDM machine that can cut these gears! ', 10, True)

            _module = inputs.addValueInput('module', 'Gear Module', _units, adsk.core.ValueInput.createByReal(float(0.5)))
            _nrteeth = inputs.addValueInput('nrteeth', 'Number of teeth', _units, adsk.core.ValueInput.createByReal(float(50)))
            _thickness = inputs.addValueInput('thickness', 'Gear Thickness [mm]', _units, adsk.core.ValueInput.createByReal(float(5.0)))
            _outsidediameter = inputs.addValueInput('outside diameter', 'Outside Diameter [mm]', _units, adsk.core.ValueInput.createByReal(float(40.0)))
            _pressureangle = inputs.addValueInput('outside diameter', 'Pressure Angle [degrees]', _units, adsk.core.ValueInput.createByReal(float(20.0)))

            
            # Connect to the command related events.
            onExecute = GearCommandExecuteHandler()
            cmd.execute.add(onExecute)
            _handlers.append(onExecute)        
            
            onInputChanged = GearCommandInputChangedHandler()
            cmd.inputChanged.add(onInputChanged)
            _handlers.append(onInputChanged)     

            onDestroy = GearCommandDestroyHandler()
            cmd.destroy.add(onDestroy)
            _handlers.append(onDestroy)
        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


# Event handler for the execute event.
class GearCommandExecuteHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandEventArgs.cast(args)

            global drawInternalGear
            #####################################################

            module = _module.value
            nrteeth = _nrteeth.value
            thickness = _thickness.value
            outsidediameter = _outsidediameter.value
            pressureangle = _pressureangle.value

            _ui.messageBox('Drawing can take a while, depending on the number of teeth. Click OK to start...'.format(traceback.format_exc()))
            drawInternalGear(outsidediameter,nrteeth,module,pressureangle,thickness)
            

        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
        

        
# Event handler for the inputChanged event.
class GearCommandInputChangedHandler(adsk.core.InputChangedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.InputChangedEventArgs.cast(args)
            changedInput = eventArgs.input

            ###################################################################            
            #_ui.messageBox('Something changed!...'.format(traceback.format_exc()))            
            
        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


def drawInternalGear(outside_diameter,teeth,module,pressureangle,thickness):

    pitch_circle_diameter = (teeth*module)
    tip_circle_diameter = pitch_circle_diameter - 2*module
    root_circle_diameter = pitch_circle_diameter + 2.5*module
    half_tooth_width = (360 / teeth)/2
    sc=0.1 #scale factor as the fusion API works in cm units, not in mm (really?!...)
    
    
    app = adsk.core.Application.get()
    ui = app.userInterface
#
#    doc = app.documents.add(adsk.core.DocumentTypes.FusionDesignDocumentType)
    design = app.activeProduct
    design.designType = adsk.fusion.DesignTypes.ParametricDesignType

    # Create a new component by creating an occurrence.
    occs = design.rootComponent.occurrences
    mat = adsk.core.Matrix3D.create()
    newOcc = occs.addNewComponent(mat)        
    newComp = adsk.fusion.Component.cast(newOcc.component)
    
    # Create a new sketch.
    sketches = newComp.sketches
    xyPlane = newComp.xYConstructionPlane
    baseSketch = sketches.add(xyPlane)       
    
###############################################################################3
    
    
    # Draw a circle for the outside diameter.
    baseSketch.sketchCurves.sketchCircles.addByCenterRadius(adsk.core.Point3D.create(0,0,0), sc*(outside_diameter/2.0))
    
    # Create an extra sketch that contains a circle of the tip diameter.
    tipCircleSketch = sketches.add(xyPlane)
    tipCircle = tipCircleSketch.sketchCurves.sketchCircles.addByCenterRadius(adsk.core.Point3D.create(0,0,0), sc*(tip_circle_diameter/2.0))
    tipCircle.isConstruction = True
    tipCircle.isFixed = True
    tipCircleSketch.isVisible = False

    # Create an extra sketch that contains a circle of the root circle diameter.
    rootCircleSketch = sketches.add(xyPlane)
    rootCircle = rootCircleSketch.sketchCurves.sketchCircles.addByCenterRadius(adsk.core.Point3D.create(0,0,0), sc*(root_circle_diameter/2.0))
    rootCircle.isConstruction = True
    rootCircle.isFixed = True
    rootCircleSketch.isVisible = False

    # Create an extra sketch that contains a circle of the pitch circle.
    pitchCircleSketch = sketches.add(xyPlane)
    pitchCircle = pitchCircleSketch.sketchCurves.sketchCircles.addByCenterRadius(adsk.core.Point3D.create(0,0,0), sc*(pitch_circle_diameter/2.0))
    pitchCircle.isConstruction = True
    pitchCircle.isFixed = True 
    pitchCircleSketch.isVisible = False             

    basec=teeth*module*math.cos(pressureangle*math.pi/180)

    invp=[]
    splineLpoints=[[0,0],[0,0],[0,0]]
    splineRpoints=[[0,0],[0,0],[0,0]]
    invp.append(involutePoint(basec/2.0,tip_circle_diameter/2.0))
    invp.append(involutePoint(basec/2.0,pitch_circle_diameter/2.0))
    invp.append(involutePoint(basec/2.0,root_circle_diameter/2.0))
    
    xp=involutePoint(basec,pitch_circle_diameter)[0]
    yp=involutePoint(basec,pitch_circle_diameter)[1]
    ang1=math.tan(yp/xp)*180/math.pi
    
    splineLpoints[0]=rotatePoint(invp[0],-ang1)
    splineLpoints[1]=rotatePoint(invp[1],-ang1)
    splineLpoints[2]=rotatePoint(invp[2],-ang1)
 
    splineRpoints[0]=[splineLpoints[0][0],-splineLpoints[0][1]]
    splineRpoints[1]=[splineLpoints[1][0],-splineLpoints[1][1]]
    splineRpoints[2]=[splineLpoints[2][0],-splineLpoints[2][1]]
    
    # Draw all the teeth
    for i in range(0, int(teeth)):            
    
        ang2=math.pi/(2*teeth)*(180/math.pi)
        
        rot_splineLpoints=[[0,0],[0,0],[0,0]]
        rot_splineLpoints[0]=rotatePoint(splineLpoints[0],-ang2 + (360/teeth)*i)
        rot_splineLpoints[1]=rotatePoint(splineLpoints[1],-ang2 + (360/teeth)*i)
        rot_splineLpoints[2]=rotatePoint(splineLpoints[2],-ang2 + (360/teeth)*i)
        
        rot_splineRpoints=[[0,0],[0,0],[0,0]]
        rot_splineRpoints[0]=rotatePoint(splineRpoints[0],ang2 + (360/teeth)*i)
        rot_splineRpoints[1]=rotatePoint(splineRpoints[1],ang2 + (360/teeth)*i)
        rot_splineRpoints[2]=rotatePoint(splineRpoints[2],ang2 + (360/teeth)*i)
        
        pointsubset2=adsk.core.ObjectCollection.create()
        pointsubset2.add(adsk.core.Point3D.create(sc*rot_splineLpoints[0][0], sc*rot_splineLpoints[0][1], 0))
        pointsubset2.add(adsk.core.Point3D.create(sc*rot_splineLpoints[1][0], sc*rot_splineLpoints[1][1], 0))
        pointsubset2.add(adsk.core.Point3D.create(sc*rot_splineLpoints[2][0], sc*rot_splineLpoints[2][1], 0))
        baseSketch.sketchCurves.sketchFittedSplines.add(pointsubset2)
    
        pointsubset=adsk.core.ObjectCollection.create()
        pointsubset.add(adsk.core.Point3D.create(sc*rot_splineRpoints[0][0], sc*rot_splineRpoints[0][1], 0))
        pointsubset.add(adsk.core.Point3D.create(sc*rot_splineRpoints[1][0], sc*rot_splineRpoints[1][1], 0))
        pointsubset.add(adsk.core.Point3D.create(sc*rot_splineRpoints[2][0], sc*rot_splineRpoints[2][1], 0))
        baseSketch.sketchCurves.sketchFittedSplines.add(pointsubset)
        
        # Draw the inner arc for the tooth.
        pt1=rotatePoint([rot_splineLpoints[0][0],rot_splineLpoints[0][1]],360.0/teeth)
        pt2=rotatePoint([tip_circle_diameter/2,0],(360.0/teeth)/2 +i*360/teeth)
        midPoint1 = adsk.core.Point3D.create(sc*pt2[0],sc*pt2[1], 0)
        endPoint1 = adsk.core.Point3D.create(sc*pt1[0],sc*pt1[1], 0)
        beginPoint1 = adsk.core.Point3D.create(sc*rot_splineRpoints[0][0],sc*rot_splineRpoints[0][1], 0)
        baseSketch.sketchCurves.sketchArcs.addByThreePoints(beginPoint1, midPoint1, endPoint1)
        
        # Draw the outer arc for the tooth.
        pt3=rotatePoint([root_circle_diameter/2,0],(360/teeth)*i)
        midPoint2 = adsk.core.Point3D.create(sc*pt3[0],sc*pt3[1],0)
        endPoint2 = adsk.core.Point3D.create(sc*rot_splineLpoints[2][0],sc*rot_splineLpoints[2][1], 0)
        beginPoint2 = adsk.core.Point3D.create(sc*rot_splineRpoints[2][0],sc*rot_splineRpoints[2][1], 0)
        baseSketch.sketchCurves.sketchArcs.addByThreePoints(beginPoint2, midPoint2, endPoint2)
        
#        # Get the profile defined by the circle.
    prof = baseSketch.profiles.item(0)

    # Create an extrusion input to be able to define the input needed for an extrusion
    # while specifying the profile and that a new component is to be created
    
    #extrudes = rootComp.features.extrudeFeatures
    extrudes = newComp.features.extrudeFeatures    
    extInput = extrudes.createInput(prof, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)

    # Define that the extent is a distance extent of 5 cm.
    distance = adsk.core.ValueInput.createByReal(thickness/10.0)
    extInput.setDistanceExtent(False, distance)
    newComp.name = 'Internal Gear'

    # Create the extrusion.
    ext = extrudes.add(extInput)    
    

def involutePoint(baseCircleRadius, distFromCenterToInvolutePoint):
        # Calculate the other side of the right-angle triangle defined by the base circle and the current distance radius.
        # This is also the length of the involute chord as it comes off of the base circle.
        triangleSide = math.sqrt(math.pow(distFromCenterToInvolutePoint,2) - math.pow(baseCircleRadius,2)) 
        
        # Calculate the angle of the involute.
        alpha = triangleSide / baseCircleRadius

        # Calculate the angle where the current involute point is.
        theta = alpha - math.acos(baseCircleRadius / distFromCenterToInvolutePoint)

        # Calculate the coordinates of the involute point.    
        x = distFromCenterToInvolutePoint * math.cos(theta)
        y = distFromCenterToInvolutePoint * math.sin(theta)

        inv_point=[x,y]
        # Create a point to return.        
        return inv_point

def rotatePoint(point, angle):
    point_ret=[0.0,0.0]
    rang=angle*(math.pi/180)
    cosA = math.cos(rang)
    sinA = math.sin(rang)
    point_ret=[point[0]*cosA - point[1]*sinA,point[0]*sinA + point[1]*cosA]
    return point_ret
      